(function ($) {
  Drupal.behaviors.media_crop = {
    attach: function (context) {
      var freeze = false;

      var container = $('div.media-item', context);

      var origWidth;
      var origHeight;
      var imageScale;

      var cropEnabled = false;

      var getImageScale = function () {
        return 350 / Math.max(origWidth, origHeight);
      };

      var calculateAnchors = function (anchor, orig, crop) {
        var ret = anchor;
        switch (anchor) {
          case "top":
          case "left":
            ret = 0;
            break;
          case "bottom":
          case "right":
            ret = orig - crop;
            break;
          case "center":
            ret = (orig / 2) - (crop / 2);
            break;
        }

        return ret;
      };

      var areaselectoptions = {
        handles: true,
        onSelectEnd: function (img, selection) {
          $('input[name=x]', context).val(selection.x1);
          $('input[name=y]', context).val(selection.y1);
          $('input[name=width]', context).val(selection.width);
          $('input[name=height]', context).val(selection.height);
        }
      };

      var setMargins = function (img) {
        var w = img.width();
        var h = img.height();

        // Resize the image if it is not in the correct size
        if (w != 350 || h != 350) {
          var normalizer = Math.max(w, h);
          w = Math.floor(w / normalizer * 350);
          h = Math.floor(h / normalizer * 350);
          img.width(w);
          img.height(h);
        }

        // add left margin for the image
        if (w < 350) {
          img.css('margin-left', (350 - w) / 2);
        }
      };

      // This killswitch is needed because of Internet Explorer.
      // For some weird reason, Internet Explorer's favourite hobby is to
      // run the onLoad event handler. This killswitch makes sure that it
      // runs exactly once.
      var killswitch = false;
      var img = $('div.media-item img')
        .load(function () {
          if (!killswitch) {
            // Determining the original width and height of the image.
            $('<img/>')
              .load(function () {
                origWidth = this.width;
                origHeight = this.height;
                imageScale = getImageScale();
                resetDimensions();
                setMargins(img);
                updateRotation(0, null);
                killswitch = true;
              })
              .attr('src', $(this).attr('src'));
          }
        });

      $('div.rotated-images img.rotated')
        .load(function () {
          setMargins($(this));
        });

      var resetDimensions = function () {
        $('input[name=x]', context).val('');
        $('input[name=y]', context).val('');
        $('input[name=width]', context).val('');
        $('input[name=height]', context).val('');
      };

      var updateRotation = function (angle, finished) {
        freeze = true;
        var rot_input = $('input[name=rotate]', context);
        var rot = Number(rot_input.val());
        var oldrot = rot;
        var rotatedImage;

        rot = (rot + angle) % 360;
        if (rot < 0) {
          rot = 360 + rot;
        }

        rot_input.val(rot);

        var rotDiff = Math.abs(oldrot - rot);

        rotatedImage = $('img.rotated-' + rot, context);
        var currentimage = rotatedImage.length > 0 ? rotatedImage : img;

        var animationFinished = function () {
          img
            .show()
            .attr('class', img
              .attr('class')
              .replace(/\bimage-rotate-[-0-9]+-[-0-9]+\b/, ''))
            .addClass('image-rotate-' + oldrot + '-' + rot)
            .imgAreaSelect({hide: true, remove: true});
        };

        var animationOptions = {
          duration: 50,
          easing: 'linear',
          queue: false
        };

        setMargins(currentimage);
        var cih = currentimage.height();

        if (rotDiff) {
          // Resizing the container
          currentimage.animate({
            'margin-top': (350 - cih) / 2
          }, $.extend({
            complete: function () {
              img.css('margin-top', (350 - img.height()) / 2);
            }
          }, animationOptions));
          container.animate({
            height: 350
          }, animationOptions);
          setTimeout(animationFinished, animationOptions.duration);
        }

        // Saves the selection
        var x = Number($('input[name=x]', context).val());
        var y = Number($('input[name=y]', context).val());
        var w = Number($('input[name=width]', context).val());
        var h = Number($('input[name=height]', context).val());

        if (rotDiff) {
          // Moves back the rotated images.
          $('div.media-item img.rotated', context)
            .imgAreaSelect({hide: true, remove: true})
            .appendTo($('div.rotated-images'));
        }

        setTimeout(function () {
          if (rot != 0 && rotDiff) {
            // Replaces the original image with a rotated.
            img
              .hide()
              .after(rotatedImage);
          }

          var selectionData = {};
          var show = false;

          if (x > 0 || y > 0 || w > 0 || h > 0) {
            var iw = currentimage.width();
            var ih = currentimage.height();
            var k, x_, y_, w_, h_;
            switch (rotDiff) {
              case 270:
              case 90:
                // Restores the selection
                k = angle / 90; // k will be either -1 or 1
                if (rotDiff == 270) {
                  k /= -3;
                }
                x_ = k > 0 ? (iw - (h + y)) : y;
                y_ = k > 0 ? x : (ih - (w + x));
                w_ = h;
                h_ = w;
                break;
              case 0:
                x_ = x;
                y_ = y;
                w_ = w;
                h_ = h;
                break;
            }

            selectionData = {
              x1: x_,
              x2: x_ + w_,
              y1: y_,
              y2: y_ + h_
            };

            show = true;
          }

          if (!show) {
            freeze = false;
          }

          container.animate({
            height: cih
          }, animationOptions);
          currentimage.animate({
            'margin-top': 0
          }, animationOptions);

          setTimeout(function () {
            currentimage.imgAreaSelect(
              $.extend({}, areaselectoptions,
                       {show: show, enable: cropEnabled}, selectionData));
            if (show) {
              $('input[name=x]').val(x_);
              $('input[name=y]').val(y_);
              $('input[name=width]').val(w_);
              $('input[name=height]').val(h_);
            }
            freeze = false;
            if (finished) {
              finished();
            }
          }, rotDiff ? 500 : 0);

        }, rotDiff ? 350 : 0);
      };

      $('select[name=format]', context).change(function () {
        var selected = $(this).val();
        var resetUI = function () {
          resetDimensions();
          updateRotation(360 - Number($('input[name=rotate]').val()), null);
        };
        try {
          if (Drupal.settings.media_crop.styles[selected].safe) {
            var props = Drupal.settings.media_crop.styles[selected];
            resetDimensions();
            $('input[name=rotate]').val(0);
            var w = (props.rotation / 90) & 1 ? img.height() : img.width();
            var h = (props.rotation / 90) & 1 ? img.width() : img.height();
            updateRotation(props.rotation, function () {
              if (props.crop) {
                var anchor = props.crop.anchor.split('-');
                $('input[name=x]', context).val(
                  calculateAnchors(anchor[0], w, props.crop.width));
                $('input[name=y]', context).val(
                  calculateAnchors(anchor[1], h, props.crop.height));
                $('input[name=width]', context).val(props.crop.width);
                $('input[name=height]', context).val(props.crop.height);
                updateRotation(0, null);
              }
            });
          }
          else {
            resetUI();
          }
        }
        catch (e) {
          resetUI();
        }
      });

      $('body').delegate('div.imgareaselect-outer', 'click', function () {
        if (cropEnabled) {
          resetDimensions();
        }
      });

      $('.rotate-left-button:not(.processed)')
        .addClass('processed')
        .click(function () {
           if (!freeze) {
             updateRotation(-90, null);
           }
        });

      $('.rotate-right-button:not(.processed)')
        .addClass('processed')
        .click(function () {
           if (!freeze) {
             updateRotation(90, null);
           }
        });

      $('.crop-button:not(.processed)')
        .addClass('processed')
        .click(function () {
           $(this).hide();
           $('.crop-active-button').show();
           cropEnabled = true;
           $('.media-item img:visible')
             .imgAreaSelect({enable: true});
         });

      $('.crop-active-button:not(.processed)')
        .addClass('processed')
        .click(function () {
           $(this).hide();
           $('.crop-button').show();
           cropEnabled = false;
           $('.media-item img:visible')
             .imgAreaSelect({enable: false});
         });

      var getFormattedMedia = Drupal.media.formatForm.getFormattedMedia;
      Drupal.media.formatForm.getFormattedMedia = function () {
        var mediadata = getFormattedMedia();
        // Override the src of the image.
        // I use regex here, because using jQuery to manipulate the
        // "src" attribute will cause a bug. If I create a jQuery object here,
        // it will start a request, which gets cancelled when this
        // iframe closes (which is just a few lines of JS after this function
        // returns). Because of this, an empty image will appear in
        // the WYSIWYG editor.

        mediadata.html = mediadata.html.replace(
                /http[s]?:\/\/[a-zA-Z0-9\.\/%~_-]+/,
          Drupal.settings.basePath +
          'media/image/' + mediadata.type.replace(/^media_/, '') + '/' +
          ($('input[name=fid]', context).val() || '0') + '/' +
          ($('input[name=rotate]', context).val() || '0') + '/' +
          ($('input[name=x]', context).val() || '0') + '/' +
          ($('input[name=y]', context).val() || '0') + '/' +
          ($('input[name=width]', context).val() || '0') + '/' +
          ($('input[name=height]', context).val() || '0'));

        return mediadata;
      };
    }
  };
})(jQuery);
